import { Given, When, Then } from '@badeball/cypress-cucumber-preprocessor';

let directory_id;
let tagset_ids = [];
let sub_tagset_ids = [];

Given('There is one directory with other directories and docs', () => {
  cy.appFactories([['create_list', 'tagset_doc', 2, 'with_informed', 'with_file']]).then(records => {
    records[0].forEach(record => {
      tagset_ids.push(record.id);
    });
  });

  cy.appFactories([['create', 'tagset_doc']]).then(records => {
    tagset_ids.push(records[0].id);
  });

  cy.appFactories([['create_list', 'tagset_doc', 2, 'with_informed']]).then(records => {
    records[0].forEach(record => {
      sub_tagset_ids.push(record.id);
    });
  });

  cy.appFactories([['create', 'tagset_directory', { responsible: '1', tagset_ids: sub_tagset_ids }]]).then(records => {
    tagset_ids.push(records[0].id);
  });

  cy.appFactories([['create', 'tagset_directory', 'with_logs', { responsible: '1', id: '1', tagset_ids: tagset_ids }]]).then(records => {
    directory_id = records[0].id;
  });
});

Given('There is a directory with some docs', () => {
  cy.appFactories([['create_list', 'tagset_doc', 5]]).then(records => {
    records[0].forEach(record => {
      tagset_ids.push(record.id);
    });
  });
  cy.appFactories([['create', 'tagset_directory', 'with_logs', { responsible: '1', id: '1', tagset_ids: tagset_ids }]]).then(records => {
    directory_id = records[0].id;
  });
});

Given('There is a directory with a lot of docs', () => {
  cy.appFactories([['create_list', 'tagset_doc', 20]]).then(records => {
    records[0].forEach(record => {
      tagset_ids.push(record.id);
    });
  });
  cy.appFactories([['create', 'tagset_directory', 'with_logs', { responsible: '1', id: '1', tagset_ids: tagset_ids }]]).then(records => {
    directory_id = records[0].id;
  });
});

When('I visit the directory url', () => {
  cy.visit(`/spirit_hub/directories/${directory_id}/index`);
});

Then('I should see the right columns', () => {
  cy.get('#model_table td').eq(3).contains('Doc Title 1');
  cy.get('#model_table td').eq(4).contains('298.15 Ko');
  cy.get('#model_table td').eq(5).contains(new Date().toLocaleDateString('fr-FR'));
  cy.get('#model_table td').eq(6).find('div').should('exist');
  cy.get('#model_table td').eq(7).find('img').should('exist');
  cy.get('#model_table td').eq(8).contains('À traiter');
  cy.get('#model_table td').eq(9).find('a').should('exist');
});

When('I click on the first directory', () => {
  cy.get('[data-controller="toggle-subrows"]').first().click();
});

Then('I should see sub documents', () => {
  cy.get('#model_table tbody tr:visible').should('have.length', 6);
});

Then('I should not see sub documents', () => {
  cy.get('#model_table tbody tr:visible').should('have.length', 4);
});

// Add/Remove doc to completeness
Given('There is a directory with some docs, directories and completness', () => {
  cy.appFactories([['create_list', 'tagset_doc', 3]]).then(records => {
    records[0].forEach(record => {
      tagset_ids.push(record.id);
    });
  });

  cy.appFactories([['create', 'tagset_directory', { responsible: '1' }]]).then(records => {
    tagset_ids.push(records[0].id);
    cy.appFactories([['create', 'completeness', { tagset_id: tagset_ids.at(-1), title: 'Completness subdirectory 1' }]]);
    cy.appFactories([['create', 'completeness', { tagset_id: tagset_ids.at(-1), title: 'Completness subdirectory 2' }]]);
  });

  cy.appFactories([['create', 'tagset_directory', { responsible: '1', id: '1', tagset_ids: tagset_ids }]]).then(records => {
    directory_id = records[0].id;
    cy.appFactories([['create', 'completeness', { tagset_id: directory_id, title: 'Completness directory' }]]);
  });
});

When('I can add a doc to directory completeness', () => {
  cy.get('#model_table tbody tr:visible:last button:last').click();
  cy.get('input[data-add-doc-to-x-target="fileInput"]').attachFile('files/example.pdf');
  cy.get('form[data-controller="add-doc-to-x"] button[type="submit"]').click();
  cy.waitForTurboResponse();
  cy.get('#model_table tbody tr:visible').last().should('not.contain', 'À completer');
  cy.get('#tagset_1_global_avancement').contains('50 %');
});

When('I can add a doc to subdirectory completeness', () => {
  cy.get('#model_table tbody tr:visible').eq(-2).find('img:first').click();
  cy.get('#model_table tbody tr:visible').eq(-2).find('button:last').click();
  cy.get('input[data-add-doc-to-x-target="fileInput"]').attachFile('files/example.pdf');
  cy.get('form[data-controller="add-doc-to-x"] button[type="submit"]').click();
  cy.waitForTurboResponse();
  cy.get('#model_table tbody tr:visible').eq(-4).should('contain', 'À completer');
  cy.get('#model_table tbody tr:visible').eq(-4).should('contain', '50 %');
  cy.get('#tagset_1_global_avancement').contains('75 %');
});

When('I can remove a doc from directory completeness', () => {
  cy.get('#model_table tbody tr:visible').last().find('button:last').click();
  cy.waitForTurboResponse();
  cy.get('#model_table tbody tr:visible').last().should('contain', 'A compléter');
  cy.get('#tagset_1_global_avancement').contains('25 %');
});

When('I can remove a doc from subdirectory completeness', () => {
  cy.get('#model_table tbody tr:visible').eq(-2).find('button:last').click();
  cy.waitForTurboResponse();
  cy.get('#tagset_1_global_avancement').contains(' 0 %');
  cy.get('#model_table tbody tr:visible').eq(-2).should('contain', 'A compléter');
  cy.get('#model_table tbody tr:visible').eq(-4).should('contain', ' 0 %');
});

When('I can add multiple docs to directory completeness', () => {
  cy.get('#model_table tbody tr:visible:last button:last').click();
  cy.get('input[data-add-doc-to-x-target="fileInput"]').attachFile(['files/example.pdf', 'files/example.jpeg']);
  cy.get('form[data-controller="add-doc-to-x"] button[type="submit"]').click();
  cy.waitForTurboResponse();
  cy.get('#model_table tbody tr:visible').eq(-2).should('contain', 'example.pdf');
  cy.get('#model_table tbody tr:visible').eq(-1).should('contain', 'example.jpeg');
});

// Add/Remove doc to tagset_doc
Given('This is some tagset_doc with file', () => {
  cy.appFactories([['create', 'tagset_doc', 'with_file']]).then(records => {
    tagset_ids.push(records[0].id);
  });

  cy.appFactories([['create', 'tagset_doc', 'with_file']]).then(records => {
    cy.appFactories([['create', 'tagset_directory', { responsible: '1', tagset_ids: [records[0].id] }]]).then(records => {
      tagset_ids.push(records[0].id);
    });
  });

  cy.appFactories([['create', 'tagset_directory', { responsible: '1', id: '1', tagset_ids: tagset_ids }]]).then(records => {
    directory_id = records[0].id;
  });
});

When('I can add a doc to tagset_doc', () => {
  cy.get('#model_table tbody tr:visible:first button:eq(3)').click();
  cy.get('input[data-add-doc-to-x-target="fileInput"]').attachFile('files/example.pdf');
  cy.get('form[data-controller="add-doc-to-x"] button[type="submit"]').click();
  cy.waitForTurboResponse();
  cy.get('#model_table tbody tr:visible').eq(0).should('contain', 'Doc Title 1');
  cy.get('#model_table tbody tr:visible').eq(1).should('contain', 'Doc Title 1');
  cy.get('#model_table tbody tr:visible').eq(2).should('contain', 'example.pdf');
});

When('I can remove a doc to tagset_doc', () => {
  cy.get('#model_table tbody tr:visible').eq(1).find('button:last').click();
});

When('I can add multiple doc to tagset_doc', () => {
  cy.get('#model_table tbody tr:visible:last [data-controller="toggle-subrows"]').first().click();
  cy.get('#model_table tbody tr:visible:last button:eq(-2)').click();
  cy.get('input[data-add-doc-to-x-target="fileInput"]').attachFile(['files/example.pdf', 'files/example.jpeg']);
  cy.get('form[data-controller="add-doc-to-x"] button[type="submit"]').click();
  cy.waitForTurboResponse();
  cy.get('#model_table tbody tr:visible').eq(-1).should('contain', 'example.jpeg');
  cy.get('#model_table tbody tr:visible').eq(-2).should('contain', 'example.pdf');
  cy.get('#model_table tbody tr:visible').eq(-3).should('contain', 'Doc Title 2');
});

When('I can remove a doc from multiple tagset_doc', () => {
  cy.get('#model_table tbody tr:visible').eq(-2).find('button:last').click();
  cy.waitForTurboResponse();
  cy.get('#model_table tbody tr:visible').eq(-2).should('not.contain', 'example.pdf');
  cy.get('#model_table tbody tr:visible').should('have.length', 5);
  cy.get('#model_table tbody tr:visible').eq(-2).find('button:last').click();
  cy.waitForTurboResponse();
  cy.get('#model_table tbody tr:visible').should('have.length', 3);
});
